# -*- coding: utf-8 -*-
from gui import mainWindow
from PythonQt import Qt, QtGui
from RoadCentre import plugins

try:
    from RoadCentre import project
except ImportError:
    project = None


class Filter:
    TOOLBAR_OBJ_NAME = "toolBarObjfilter"
    NO_DT_TITLE = "Без детекторов"
    FILTER_BY_TYPE = 1
    FILTER_BY_DT = 2
    FILTER_CUSTOM = 3

    def __init__(self):
        self.toolbar = mainWindow.addToolBar('Фильтр объектов')
        self.toolbar.objectName = Filter.TOOLBAR_OBJ_NAME
        self.menu = QtGui.QMenu(self.toolbar.windowTitle, self.toolbar)
        self.menu.setIcon(QtGui.QIcon('objfilter/filter.png'))
        self.action_group = Qt.QActionGroup(self.toolbar)
        self.action_group.setExclusive(0)
        self.menu.setContentsMargins(20, 5, 0, 5)
        self.action_group.connect('triggered(QAction*)', self._on_filter_triggered)
        menuView = mainWindow.findChild('QMenu', 'menuView')
        if menuView:
            self.toolbar.addAction(menuView.addMenu(self.menu))
        act = self.toolbar.addAction(QtGui.QIcon('objfilter/filter_none.png'), "Отключить фильтр")
        act.enabled = False
        if menuView:
            menuView.addAction(act)
        act.connect('triggered()', self._on_filter_none)
        act = self.toolbar.addAction(QtGui.QIcon('objfilter/filter_all.png'), "Включить все фильтры")
        act.enabled = False
        if menuView:
            menuView.addAction(act)
        act.connect('triggered()', self._on_filter_all)
        self.menu.enabled = False
        if project:
            project.connect('projectLoaded(const QString&)', self._on_project_loaded)
        self.objects_without_detectors = []
        if hasattr(plugins, 'JsonRpcClientPlugin'):
            self.rcd_rpc_obj = plugins.JsonRpcClientPlugin.getObject("rcd")
            if self.rcd_rpc_obj:
                self.rcd_rpc_obj.connect('connected(bool)', self._on_rcd_rpc_connected)
        self.custom_filters = {}

    def create_actions(self):
        if project is None:
            return
        obj_type_count = {}
#        hidden_types = set()
#        userSettings.beginGroup("obj_filter")
#        for hex_name in userSettings.childKeys():
#            if userSettings.value(hex_name)=='y':
#                continue
#            hidden_types.add(binascii.unhexlify(hex_name).decode())
#        userSettings.endGroup()
        has_new = False
        for obj in project.objects():
            type_name = self._obj_type_name(obj)
            if type_name not in obj_type_count:
                obj_type_count[type_name] = 1
                has_new = True
            else:
                obj_type_count[type_name] += 1
        if has_new > 0:
            self.clear_actions()
            for type_name in obj_type_count:
                self._add_action(type_name, obj_type_count[type_name], True, Filter.FILTER_BY_TYPE)
        self.menu.enabled = True
        for act in self.toolbar.actions():
            act.enabled = True

    def clear_actions(self):
        if self.menu is None:
            return
        for act in self.menu.actions():
            act.delete()
        self.menu.enabled = False
        for act in self.toolbar.actions():
            act.enabled = True

    def add_filter(self, name, checker_fn):
        self.custom_filters[name] = checker_fn
        obj_count = 0
        for obj in project.objects():
            if checker_fn(obj):
                obj_count += 1
        self._add_action(name, obj_count, True, Filter.FILTER_CUSTOM)

    def remove_filter(self, name):
        if name not in self.custom_filters:
            return
        del self.custom_filters[name]
        self._remove_action(name)
        self._apply_filter()

    def set_filter_checked(self, name, checked):
        for act in self.menu.actions():
            if self._act_text(act) == name:
                cb = act.defaultWidget()
                cb.checked = checked
                break

    def _on_filter_triggered(self, act):
        self._apply_filter()

    def _add_action(self, name, obj_count, checked, data):
        if self.action_group is None or self.menu is None:
            return
        wact = None
        for act in self.menu.actions():
            if self._act_text(act) == name:
                wact = act
                break
        cb_text = name + " (%d)" % obj_count
        if wact is None:
            wact = QtGui.QWidgetAction(self.action_group)
            cb = QtGui.QCheckBox(cb_text, self.menu)
            cb.checked = checked
            cb.connect('toggled(bool)', wact.triggered)
            wact.setDefaultWidget(cb)
            wact.text = name
            wact.setData(data)
            self.action_group.addAction(wact)
            self.menu.addAction(wact)
        else:
            act.defaultWidget().text = cb_text

    def _remove_action(self, name):
        for act in self.menu.actions():
            if self._act_text(act) == name:
                act.delete()
                break

    def _apply_filter(self):
        visible_types = []
        no_dt_obj_visible = True
        unchecked_custom = []
        for act in self.menu.actions():
            if act.data() == Filter.FILTER_BY_TYPE:
                if act.defaultWidget().checked:
                    visible_types.append(self._act_text(act))
            elif act.data() == Filter.FILTER_BY_DT:
                no_dt_obj_visible = act.defaultWidget().checked
            elif act.data() == Filter.FILTER_CUSTOM:
                checker = self.custom_filters.get(self._act_text(act))
                if checker and not act.defaultWidget().checked:
                    unchecked_custom.append(checker)
        for obj in project.objects():
            obj_visible = True
            if not no_dt_obj_visible:
                obj_visible = obj.id not in self.objects_without_detectors
            if obj_visible:
                obj_visible &= self._obj_type_name(obj) in visible_types
            if obj_visible:
                for checker in unchecked_custom:
                    obj_visible &= checker(obj)
                    if not obj_visible:
                        break
            obj.graphicsItem.setVisible(obj_visible)

    def _on_project_loaded(self, file_name):
        self._apply_filter()

    def _on_filter_none(self):
        for act in self.menu.actions():
            act.defaultWidget().checked = True

    def _on_filter_all(self):
        for act in self.menu.actions():
            act.defaultWidget().checked = False

    def _obj_type_name(self, obj):
        type_name = obj.type
        if len(type_name) == 0:
            type_name = "Неизвестные"
        return type_name

    def _on_rcd_rpc_connected(self, connected):
        if connected:
            rpc_resp = self.rcd_rpc_obj.postCommand("rcdService.objectsWithDetectors")
            rpc_resp.connect('responseFinished(Response*)', self._on_obj_with_dt_answer)
        elif len(self.objects_without_detectors) > 0:
            self.objects_without_detectors = []
            self._remove_action(Filter.NO_DT_TITLE)
            self._apply_filter()

    def _on_obj_with_dt_answer(self, resp):
        dt_obj_list = resp.answer().get("result",[])
        for obj in project.objects():
            if obj.id not in dt_obj_list:
                self.objects_without_detectors.append(obj.id)
        obj_count = len(self.objects_without_detectors)
        if obj_count > 0:
            self._add_action(Filter.NO_DT_TITLE, obj_count, True, Filter.FILTER_BY_DT)
        else:
            self._remove_action(Filter.NO_DT_TITLE)
    
    def _act_text(self, act):
        return act.text.replace('&','')

filter = None


def on_init():
    global filter
    filter = Filter()


def on_popen():
    if filter:
        filter.create_actions()


def on_pclose():
    if filter:
        filter.clear_actions()


def add_filter(name, checker_fn):
    if filter is None:
        return
    filter.add_filter(name, checker_fn)


def remove_filter(name):
    if filter is None:
        return
    filter.remove_filter(name)


def set_filter_checked(name, checked):
    if filter is None:
        return
    filter.set_filter_checked(name, checked)
